# SPDX-License-Identifier: MIT
import os
import pytest
from neo_airpress.compressor import PKPass


@pytest.fixture
def pkpass():
    return PKPass()


@pytest.fixture
def pkpass_with_assets():
    p = PKPass(
        ("icon.png", b"00001111"),
        ("pass.json", b"11110000"),
    )
    return p


@pytest.fixture
def key():
    with open(
        os.path.join(
            os.path.dirname(__file__), "credentials/unprotected_dummy_key.pem"
        ),
        "rb",
    ) as k:
        key = k.read()
    return key


@pytest.fixture
def cert():
    with open(
        os.path.join(os.path.dirname(__file__), "credentials/unprotected_dummy_cert.pem"),
        "rb",
    ) as c:
        cert = c.read()
    return cert


@pytest.fixture
def wwdr_cert():
    with open(
        os.path.join(os.path.dirname(__file__), "credentials/AppleWWDRCA.pem"),
        "rb",
    ) as c:
        cert = c.read()
    return cert



# SPDX-License-Identifier: MIT
import pytest


def test_should_add_valid_asset_to_pkpass_using_helper_method(pkpass):
    asset = ("pass.json", b"00001111")
    pkpass.add_to_pass_package(asset)
    assert pkpass["pass.json"]


def test_should_add_valid_asset_to_pkpass_using_magic_method(pkpass):
    pkpass["pass.json"] = b"00001111"
    assert pkpass["pass.json"]


def test_should_retrieve_asset_from_pkpass(pkpass):
    asset_name = "logo.png"
    asset_content = b"00001111"
    pkpass[asset_name] = asset_content

    retrieved_asset = pkpass[asset_name]

    assert retrieved_asset == asset_content


def test_should_remove_asset_from_pkpass(pkpass):
    asset_name = "logo.png"
    asset_content = b"11110000"
    pkpass[asset_name] = asset_content

    del pkpass[asset_name]

    with pytest.raises(KeyError):
        _ = pkpass[asset_name]


def test_should_raise_key_error_removing_non_existing_asset_from_pkpass(pkpass):
    with pytest.raises(KeyError):
        del pkpass["bobbish.xyz"]


def test_should_raise_type_error_adding_asset_with_wrong_name_type(pkpass):
    asset = ("pass.json".encode(), b"11110000")
    with pytest.raises(TypeError):
        pkpass.add_to_pass_package(asset)


def test_should_raise_type_error_adding_asset_with_wrong_content_type(pkpass):
    asset = ("pass.json", object())
    with pytest.raises(TypeError):
        pkpass.add_to_pass_package(asset)


def test_should_raise_assertion_error_for_asset_with_empty_content(pkpass):
    asset = ("pass.json", b"")
    with pytest.raises(AssertionError):
        pkpass.add_to_pass_package(asset)


def test_should_raise_assertion_error_adding_unsupported_asset(pkpass):
    asset = ("unknown.doc", b"11000011")
    with pytest.raises(AssertionError):
        pkpass.add_to_pass_package(asset)


def test_should_explicitly_add_unsupported_asset(pkpass):
    asset = ("unknown.doc", b"11000011")
    pkpass.add_to_pass_package(asset, validate=False)



# SPDX-License-Identifier: MIT
import pytest
from neo_airpress.compressor import PKPass


def test_should_initialize_pkpass_without_assets():
    pkpass = PKPass()
    assert pkpass


def test_should_initialize_pkpass_with_assets():
    pkpass = PKPass(
        ("icon.png", b"00001111"),
        ("pass.json", b"11110000"),
    )
    assert pkpass["icon.png"]
    assert pkpass["pass.json"]


def test_should_raise_type_error_during_initialization_with_assets_of_wrong_type():
    with pytest.raises(TypeError):
        _ = PKPass(("pass.json", object()))


def test_should_raise_assertion_error_during_initialization_with_unsupported_asset():
    with pytest.raises(AssertionError):
        _ = PKPass(("unknown.doc", b"11001100"))


def test_should_explicitly_initialize_pkpass_with_unsupported_asset():
    pkpass = PKPass(("unknown.doc", b"11001100"), validate=False)
    assert pkpass["unknown.doc"]



# SPDX-License-Identifier: MIT
import pytest



def test_should_raise_attribute_error_trying_to_access_signature_of_unsigned_pkpass(
    pkpass_with_assets,
):
    with pytest.raises(AttributeError):
        _ = pkpass_with_assets.signature


def test_should_raise_assertion_error_trying_to_sign_pkpass_without_credentials(
    pkpass_with_assets,
):
    with pytest.raises(AssertionError):
        pkpass_with_assets.sign()


def test_should_raise_assertion_error_trying_to_sign_pkpass_without_certificate(pkpass_with_assets, key):
    with pytest.raises(AssertionError):
        pkpass_with_assets.sign(key=key)


def test_should_raise_assertion_error_trying_to_sign_pkpass_without_key(pkpass_with_assets, cert):
    with pytest.raises(AssertionError):
        pkpass_with_assets.sign(key=cert)


def test_should_raise_assertion_error_trying_to_sign_pkpass_without_wwdr_cert(pkpass_with_assets, cert, key):
    with pytest.raises(AssertionError):
        pkpass_with_assets.sign(cert=cert, key=key, wwdr=None)


def test_sign_method_returns_signature_as_bytes_object(pkpass_with_assets, cert, key, wwdr_cert):
    signature = pkpass_with_assets.sign(cert=cert, key=key, wwdr=wwdr_cert)
    assert signature
    assert type(signature) is bytes


def test_should_sign_pkpass_with_credentials_supplied_explicitly(pkpass_with_assets, cert, key, wwdr_cert):
    pkpass_with_assets.sign(cert=cert, key=key, wwdr=wwdr_cert)
    assert pkpass_with_assets.signature


def test_should_sign_pkpass_with_credentials_supplied_implicitly(pkpass_with_assets, cert, key, wwdr_cert):
    pkpass_with_assets.cert = cert
    pkpass_with_assets.key = key

    pkpass_with_assets.sign(wwdr=wwdr_cert)

    assert pkpass_with_assets.signature


def test_should_reset_signature_if_asset_was_added_after_signing(pkpass_with_assets, cert, key, wwdr_cert):
    pkpass_with_assets.cert = cert
    pkpass_with_assets.key = key
    pkpass_with_assets.sign(wwdr=wwdr_cert)

    pkpass_with_assets["icon@2x.png"] = b"11000011"

    with pytest.raises(AttributeError):
        _ = pkpass_with_assets.signature


def test_should_reset_signature_if_asset_was_removed_after_signing(pkpass_with_assets, cert, key, wwdr_cert):
    pkpass_with_assets["icon@2x.png"] = b"11000011"
    pkpass_with_assets.cert = cert
    pkpass_with_assets.key = key
    pkpass_with_assets.sign(wwdr=wwdr_cert)

    del pkpass_with_assets["icon@2x.png"]

    with pytest.raises(AttributeError):
        _ = pkpass_with_assets.signature



# SPDX-License-Identifier: MIT
import pytest


def test_should_raise_assertion_error_calling_manifest_without_mandatory_assets(
    pkpass,
):
    with pytest.raises(AssertionError):
        _ = pkpass.manifest_dict


def test_should_raise_assertion_error_calling_manifest_with_incomplete_pkpass(
    pkpass,
):
    pkpass.add_to_pass_package(("pass.json", b"11110000"))
    with pytest.raises(AssertionError):
        _ = pkpass.manifest_dict


def test_should_create_manifest_dictionary(pkpass):
    pkpass.add_to_pass_package(("icon.png", b"00001111"), ("pass.json", b"11110000"))

    expected_manifest_dict = {
        "icon.png": "8c4b5ab6514ff51b44d020b0006746152a53583e",
        "pass.json": "eec94a882caaf36c8840ff10ed115f63c1d4ab99",
    }

    assert isinstance(pkpass.manifest_dict, dict)
    assert pkpass.manifest_dict == expected_manifest_dict


def test_should_generate_non_empty_manifest_bytes_object(pkpass):
    pkpass.add_to_pass_package(("icon.png", b"00001111"), ("pass.json", b"11110000"))
    expected_manifest_bytes = (
        b'{\n    "icon.png":"8c4b5ab6514ff51b44d020b0006746152a53583e",'
        b'\n    "pass.json":"eec94a882caaf36c8840ff10ed115f63c1d4ab99"\n}'
    )

    assert isinstance(pkpass.manifest, bytes)
    assert pkpass.manifest == expected_manifest_bytes



# SPDX-License-Identifier: MIT
from cryptography import x509
from cryptography.hazmat.bindings.openssl.binding import Binding
from cryptography.hazmat.primitives import hashes, serialization
from .pkcs7 import PKCS7SignatureBuilder, PKCS7Options

copenssl = Binding.lib
cffi = Binding.ffi


def pkcs7_sign(
    certcontent,
    keycontent,
    wwdr_certificate,
    data,
    key_password=None,
):
    """
    Creates a PKCS#7 detached signature for the given data using provided certificates and private key.

    Args:
        certcontent (bytes): The signer's certificate in PEM format.
        keycontent (bytes): The signer's private key in PEM format.
        wwdr_certificate (bytes): Apple's Worldwide Developer Relations (WWDR) certificate in DER format.
        data (bytes): The data to be signed.
        key_password (bytes, optional): Password for the private key if it's encrypted. Defaults to None.

    Returns:
        bytes: The PKCS#7 signature in DER format.

    The function performs the following steps:
    1. Loads the signer's certificate from PEM format
    2. Loads the private key (with optional password)
    3. Loads the WWDR certificate from DER format
    4. Creates a detached PKCS#7 signature using SHA256 as the hash algorithm

    Example:
        signature = pkcs7_sign(
            cert_data,
            key_data,
            wwdr_cert_data,
            content_to_sign,
            key_password=b'optional_password'
        )
    """

    cert = x509.load_pem_x509_certificate(certcontent)
    priv_key = serialization.load_pem_private_key(keycontent, password=key_password)
    wwdr_cert = x509.load_pem_x509_certificate(wwdr_certificate)

    options = [PKCS7Options.DetachedSignature]
    return (
        PKCS7SignatureBuilder()
        .set_data(data)
        .add_signer(cert, priv_key, hashes.SHA1())
        .add_certificate(wwdr_cert)
        .sign(serialization.Encoding.DER, options)
    )



# SPDX-License-Identifier: MIT
import pytest


def test_checks_signed_pass_package_contains_assets_manifest_and_signature(pkpass_with_assets, cert, key, wwdr_cert):
    pkpass_with_assets.sign(cert=cert, key=key, wwdr=wwdr_cert)
    assert "icon.png" in pkpass_with_assets.pass_package
    assert "pass.json" in pkpass_with_assets.pass_package
    assert "manifest.json" in pkpass_with_assets.pass_package
    assert "signature" in pkpass_with_assets.pass_package


def test_should_compress_signed_pass_package_into_pkpass_archive_using_bytes_method(
    pkpass_with_assets, cert, key, wwdr_cert
):
    pkpass_with_assets.sign(cert=cert, key=key, wwdr=wwdr_cert)
    pkpass_archive = bytes(pkpass_with_assets)
    assert pkpass_archive
    assert type(pkpass_archive) is bytes


def test_should_compress_signed_pass_package_into_pkpass_archive_as_callable(pkpass_with_assets, cert, key, wwdr_cert):
    pkpass_with_assets.sign(cert=cert, key=key, wwdr=wwdr_cert)
    pkpass_archive = pkpass_with_assets()
    assert pkpass_archive
    assert type(pkpass_archive) is bytes


def test_should_fail_to_compress_pass_package_if_another_exception_occurs(
    pkpass_with_assets,
):
    with pytest.raises(Exception):
        _ = bytes(pkpass_with_assets)
